@@ -16,12 +16,61 @@ from tests import unittest
 import mock
 
 from botocore import args
+from botocore.client import ClientEndpointBridge
 from botocore.config import Config
+from botocore.hooks import HierarchicalEmitter
+from botocore.model import ServiceModel
 
 
 class TestCreateClientArgs(unittest.TestCase):
     def setUp(self):
-        self.args_create = args.ClientArgsCreator(None, None, None, None, None)
+        self.event_emitter = mock.Mock(HierarchicalEmitter)
+        self.args_create = args.ClientArgsCreator(
+            self.event_emitter, None, None, None, None)
+        self.region = 'us-west-2'
+        self.endpoint_url = 'https://ec2/'
+        self.service_model = mock.Mock(ServiceModel)
+        self.service_model.metadata = {
+            'serviceFullName': 'MyService',
+            'protocol': 'query'
+        }
+        self.service_model.operation_names = []
+        self.bridge = mock.Mock(ClientEndpointBridge)
+        self.bridge.resolve.return_value = {
+            'region_name': self.region, 'signature_version': 'v4',
+            'endpoint_url': self.endpoint_url,
+            'signing_name': 'ec2', 'signing_region': self.region,
+            'metadata': {}}
+
+    def call_get_client_args(self, **override_kwargs):
+        call_kwargs = {
+            'service_model': self.service_model,
+            'region_name': self.region,
+            'is_secure': True,
+            'endpoint_url': self.endpoint_url,
+            'verify': True,
+            'credentials': None,
+            'scoped_config': {},
+            'client_config': None,
+            'endpoint_bridge': self.bridge
+        }
+        call_kwargs.update(**override_kwargs)
+        return self.args_create.get_client_args(**call_kwargs)
+
+    def assert_create_endpoint_call(self, mock_endpoint, **override_kwargs):
+        call_kwargs = {
+            'endpoint_url': self.endpoint_url,
+            'region_name': self.region,
+            'response_parser_factory': None,
+            'timeout': (60, 60),
+            'verify': True,
+            'max_pool_connections': 10,
+            'proxies': None,
+        }
+        call_kwargs.update(**override_kwargs)
+        mock_endpoint.return_value.create_endpoint.assert_called_with(
+            self.service_model, **call_kwargs
+        )
 
     def test_compute_s3_configuration(self):
         scoped_config = {}
@@ -106,68 +155,23 @@ class TestCreateClientArgs(unittest.TestCase):
         )
 
     def test_max_pool_from_client_config_forwarded_to_endpoint_creator(self):
-        args_create = args.ClientArgsCreator(
-            mock.Mock(), None, None, None, None)
         config = botocore.config.Config(max_pool_connections=20)
-        service_model = mock.Mock()
-        service_model.metadata = {
-            'serviceFullName': 'MyService',
-            'protocol': 'query'
-        }
-        service_model.operation_names = []
-        bridge = mock.Mock()
-        bridge.resolve.return_value = {
-            'region_name': 'us-west-2', 'signature_version': 'v4',
-            'endpoint_url': 'https://ec2/',
-            'signing_name': 'ec2', 'signing_region': 'us-west-2',
-            'metadata': {}}
         with mock.patch('botocore.args.EndpointCreator') as m:
-            args_create.get_client_args(
-                service_model, 'us-west-2', True, 'https://ec2/', True,
-                None, {}, config, bridge)
-            m.return_value.create_endpoint.assert_called_with(
-                mock.ANY, endpoint_url='https://ec2/', region_name='us-west-2',
-                response_parser_factory=None, timeout=(60, 60), verify=True,
-                max_pool_connections=20, proxies=None
-            )
+            self.call_get_client_args(client_config=config)
+            self.assert_create_endpoint_call(m, max_pool_connections=20)
 
     def test_proxies_from_client_config_forwarded_to_endpoint_creator(self):
-        args_create = args.ClientArgsCreator(
-            mock.Mock(), None, None, None, None)
         proxies = {'http': 'http://foo.bar:1234',
                    'https': 'https://foo.bar:4321'}
         config = botocore.config.Config(proxies=proxies)
-        service_model = mock.Mock()
-        service_model.metadata = {
-            'serviceFullName': 'MyService',
-            'protocol': 'query'
-        }
-        service_model.operation_names = []
-        bridge = mock.Mock()
-        bridge.resolve.return_value = {
-            'region_name': 'us-west-2', 'signature_version': 'v4',
-            'endpoint_url': 'https://ec2/',
-            'signing_name': 'ec2', 'signing_region': 'us-west-2',
-            'metadata': {}}
         with mock.patch('botocore.args.EndpointCreator') as m:
-            args_create.get_client_args(
-                service_model, 'us-west-2', True, 'https://ec2/', True,
-                None, {}, config, bridge)
-            m.return_value.create_endpoint.assert_called_with(
-                mock.ANY, endpoint_url='https://ec2/', region_name='us-west-2',
-                response_parser_factory=None, timeout=(60, 60), verify=True,
-                proxies=proxies, max_pool_connections=10
-            )
+            self.call_get_client_args(client_config=config)
+            self.assert_create_endpoint_call(m, proxies=proxies)
 
     def test_s3_with_endpoint_url_still_resolves_region(self):
-        self.args_create = args.ClientArgsCreator(
-            mock.Mock(), None, None, None, None)
-        service_model = mock.Mock()
-        service_model.endpoint_prefix = 's3'
-        service_model.metadata = {'protocol': 'rest-xml'}
-        config = botocore.config.Config()
-        bridge = mock.Mock()
-        bridge.resolve.side_effect = [
+        self.service_model.endpoint_prefix = 's3'
+        self.service_model.metadata = {'protocol': 'rest-xml'}
+        self.bridge.resolve.side_effect = [
             {
                 'region_name': None, 'signature_version': 's3v4',
                 'endpoint_url': 'http://other.com/', 'signing_name': 's3',
@@ -180,26 +184,25 @@ class TestCreateClientArgs(unittest.TestCase):
                 'metadata': {}
             }
         ]
-        client_args = self.args_create.get_client_args(
-            service_model, 'us-west-2', True, 'http://other.com/', True, None,
-            {}, config, bridge)
+        client_args = self.call_get_client_args(
+            endpoint_url='http://other.com/')
         self.assertEqual(
             client_args['client_config'].region_name, 'us-west-2')
 
     def test_region_does_not_resolve_if_not_s3_and_endpoint_url_provided(self):
-        self.args_create = args.ClientArgsCreator(
-            mock.Mock(), None, None, None, None)
-        service_model = mock.Mock()
-        service_model.endpoint_prefix = 'ec2'
-        service_model.metadata = {'protocol': 'query'}
-        config = botocore.config.Config()
-        bridge = mock.Mock()
-        bridge.resolve.side_effect = [{
+        self.service_model.endpoint_prefix = 'ec2'
+        self.service_model.metadata = {'protocol': 'query'}
+        self.bridge.resolve.side_effect = [{
             'region_name': None, 'signature_version': 'v4',
             'endpoint_url': 'http://other.com/', 'signing_name': 'ec2',
             'signing_region': None, 'metadata': {}
         }]
-        client_args = self.args_create.get_client_args(
-            service_model, 'us-west-2', True, 'http://other.com/', True, None,
-            {}, config, bridge)
+        client_args = self.call_get_client_args(
+            endpoint_url='http://other.com/')
         self.assertEqual(client_args['client_config'].region_name, None)
+
+    def test_provide_retry_config(self):
+        config = botocore.config.Config(retries={'max_attempts': 10})
+        client_args = self.call_get_client_args(client_config=config)
+        self.assertEqual(
+            client_args['client_config'].retries, {'max_attempts': 10})
